--- hotspot/src/os/bsd/vm/os_bsd.cpp.orig	2015-05-22 20:39:11.252410936 +0000
+++ hotspot/src/os/bsd/vm/os_bsd.cpp	2015-05-22 20:41:53.242413755 +0000
@@ -150,6 +150,7 @@
 volatile uint64_t         os::Bsd::_max_abstime   = 0;
 #else
 int (*os::Bsd::_clock_gettime)(clockid_t, struct timespec *) = NULL;
+int (*os::Bsd::_getcpuclockid)(pthread_t, clockid_t *) = NULL;
 #endif
 pthread_t os::Bsd::_main_thread;
 int os::Bsd::_page_size = -1;
@@ -1057,6 +1058,7 @@
     // yes, monotonic clock is supported
     _clock_gettime = ::clock_gettime;
   }
+  _getcpuclockid = (int (*)(pthread_t, clockid_t *))dlsym(RTLD_DEFAULT, "pthread_getcpuclockid");
 }
 #endif
 
@@ -4245,6 +4247,8 @@
 #ifdef __APPLE__
   return os::thread_cpu_time(Thread::current(), true /* user + sys */);
 #else
+  if (Bsd::_getcpuclockid != NULL)
+    return os::thread_cpu_time(Thread::current(), true /* user + sys */);
   Unimplemented();
   return 0;
 #endif
@@ -4254,6 +4258,8 @@
 #ifdef __APPLE__
   return os::thread_cpu_time(thread, true /* user + sys */);
 #else
+  if (Bsd::_getcpuclockid != NULL)
+    return os::thread_cpu_time(thread, true /* user + sys */);
   Unimplemented();
   return 0;
 #endif
@@ -4263,6 +4269,8 @@
 #ifdef __APPLE__
   return os::thread_cpu_time(Thread::current(), user_sys_cpu_time);
 #else
+  if (Bsd::_getcpuclockid != NULL)
+    return os::thread_cpu_time(Thread::current(), user_sys_cpu_time);
   Unimplemented();
   return 0;
 #endif
@@ -4289,6 +4297,24 @@
     return ((jlong)tinfo.user_time.seconds * 1000000000) + ((jlong)tinfo.user_time.microseconds * (jlong)1000);
   }
 #else
+  if (Bsd::_getcpuclockid != NULL) {
+    struct timespec tp;
+    clockid_t clockid;
+    int ret;
+
+    /*
+     * XXX This is essentially a copy of the Linux implementation,
+     *     but with fewer indirections.
+     */
+    ret = Bsd::_getcpuclockid(thread->osthread()->pthread_id(), &clockid);
+    if (ret != 0)
+      return (-1);
+    /* NB: _clock_gettime only needs to be valid for CLOCK_MONOTONIC. */
+    ret = ::clock_gettime(clockid, &tp);
+    if (ret != 0)
+      return (-1);
+    return (tp.tv_sec * NANOSECS_PER_SEC) + tp.tv_nsec;
+  }
   Unimplemented();
   return 0;
 #endif
@@ -4313,7 +4339,7 @@
 #ifdef __APPLE__
   return true;
 #else
-  return false;
+  return (Bsd::_getcpuclockid != NULL);
 #endif
 }
 
